package XML::TMX;
$XML::TMX::VERSION = '0.35';
# ABSTRACT: Perl extensions for managing TMX files

use 5.010;
use warnings;
use strict;

use parent 'Exporter';

our @ISA = 'Exporter';
our @EXPORT_OK = qw();


1;

__END__

=pod

=encoding utf-8

=head1 NAME

XML::TMX - Perl extensions for managing TMX files

=head1 VERSION

version 0.35

=head1 SYNOPSIS

   use XML::TMX;

=head1 DESCRIPTION

XML::TMX is the top level module. At the moment it does not contain
any useful code, so check sub-modules, please.

=head1 AUTHORS

=over 4

=item *

Alberto Simões <ambs@cpan.org>

=item *

José João Almeida <jj@di.uminho.pt>

=back

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2010-2017 by Projeto Natura <natura@di.uminho.pt>.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
